@@ -1,3 +1,5 @@
+//! A module for working with graphs.
+
 use std::{
     collections::{BTreeMap, HashSet},
     hash::Hash,
@@ -32,6 +34,22 @@ where
     Edge<T>: Eq + Hash,
     T: Copy + Eq + Hash + Ord,
 {
+    /// Creates an empty graph.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use spectre::graph::Graph;
+    ///
+    /// let graph: Graph<&str> = Graph::new();
+    /// ```
+    pub fn new() -> Self
+    where
+        T: Default,
+    {
+        Default::default()
+    }
+
     /// Inserts an edge into the graph.
     pub fn insert(&mut self, edge: Edge<T>) -> bool {
         let is_inserted = self.edges.insert(edge);
@@ -45,12 +63,46 @@ where
         is_inserted
     }
 
+    /// Removes an edge from the set and returns whether it was present in the set.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use spectre::edge::Edge;
+    /// use spectre::graph::Graph;
+    ///
+    /// let mut graph = Graph::new();
+    /// graph.insert(Edge::new("a", "b"));
+    ///
+    /// assert_eq!(graph.remove(&Edge::new("a", "b")), true);
+    /// assert_eq!(graph.remove(&Edge::new("a", "c")), false);
+    /// ```
+    pub fn remove(&mut self, edge: &Edge<T>) -> bool {
+        self.edges.remove(edge)
+    }
+
     /// Checks if the graph contains an edge.
     pub fn contains(&self, edge: &Edge<T>) -> bool {
         self.edges.contains(edge)
     }
 
     /// Returns the vertex count of the graph.
+    ///
+    /// This call constructs the collection of vertices from the collection of edges. This is
+    /// because the vertex set can't accurately be updated on the basis of the addition or the
+    /// removal of an edge alone.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use spectre::edge::Edge;
+    /// use spectre::graph::Graph;
+    ///
+    /// let mut graph = Graph::new();
+    /// graph.insert(Edge::new("a", "b"));
+    ///
+    /// assert_eq!(graph.vertex_count(), 2);
+    /// ```
     pub fn vertex_count(&self) -> usize {
         self.vertices_from_edges().len()
     }
@@ -62,6 +114,21 @@ where
 
     /// Computes the density of the graph, the ratio of edges with respect to the maximum possible
     /// edges.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use spectre::edge::Edge;
+    /// use spectre::graph::Graph;
+    ///
+    /// let mut graph = Graph::new();
+    ///
+    /// graph.insert(Edge::new("a", "b"));
+    /// assert_eq!(graph.density(), 1.0);
+    ///
+    /// graph.insert(Edge::new("a", "c"));
+    /// assert_eq!(graph.density(), 2.0 / 3.0);
+    /// ```
     pub fn density(&self) -> f64 {
         let vc = self.vertex_count() as f64;
         let ec = self.edge_count() as f64;
@@ -73,6 +140,22 @@ where
     }
 
     /// Constructs the adjacency matrix for this graph.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use nalgebra::dmatrix;
+    /// use spectre::edge::Edge;
+    /// use spectre::graph::Graph;
+    ///
+    /// let mut graph = Graph::new();
+    /// graph.insert(Edge::new("a", "b"));
+    /// assert_eq!(
+    ///     graph.adjacency_matrix(),
+    ///     dmatrix![0.0, 1.0;
+    ///              1.0, 0.0]
+    /// );
+    /// ```
     pub fn adjacency_matrix(&mut self) -> DMatrix<f64> {
         // Check the cache.
         if let Some(matrix) = self.adjacency_matrix.clone() {
@@ -106,6 +189,22 @@ where
     }
 
     /// Constructs the degree matrix for this graph.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use nalgebra::dmatrix;
+    /// use spectre::edge::Edge;
+    /// use spectre::graph::Graph;
+    ///
+    /// let mut graph = Graph::new();
+    /// graph.insert(Edge::new("a", "b"));
+    /// assert_eq!(
+    ///     graph.degree_matrix(),
+    ///     dmatrix![1.0, 0.0;
+    ///              0.0, 1.0]
+    /// );
+    /// ```
     pub fn degree_matrix(&mut self) -> DMatrix<f64> {
         // Check the cache.
         if let Some(matrix) = self.degree_matrix.clone() {
@@ -132,6 +231,22 @@ where
     }
 
     /// Constructs the laplacian matrix for this graph.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use nalgebra::dmatrix;
+    /// use spectre::edge::Edge;
+    /// use spectre::graph::Graph;
+    ///
+    /// let mut graph = Graph::new();
+    /// graph.insert(Edge::new("a", "b"));
+    /// assert_eq!(
+    ///     graph.laplacian_matrix(),
+    ///     dmatrix![1.0, -1.0;
+    ///              -1.0, 1.0]
+    /// );
+    /// ```
     pub fn laplacian_matrix(&mut self) -> DMatrix<f64> {
         // Check the cache.
         if let Some(matrix) = self.laplacian_matrix.clone() {
@@ -152,6 +267,19 @@ where
     /// Returns the difference between the highest and lowest degree centrality in the network.
     ///
     /// Returns an `f64`, though the value should be a natural number.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use spectre::edge::Edge;
+    /// use spectre::graph::Graph;
+    ///
+    /// let mut graph = Graph::new();
+    /// graph.insert(Edge::new("a", "b"));
+    /// graph.insert(Edge::new("a", "c"));
+    ///
+    /// assert_eq!(graph.degree_centrality_delta(), 1.0);
+    /// ```
     pub fn degree_centrality_delta(&mut self) -> f64 {
         let degree_matrix = self.degree_matrix();
 
@@ -191,8 +319,8 @@ where
         let eigenvalue_vector_pairs = sorted_eigenvalue_vector_pairs(adjacency_matrix, ascending);
         let (_highest_eigenvalue, highest_eigenvector) = &eigenvalue_vector_pairs[0];
 
-        // The eigenvector is a relative score of node importance (normalised by the norm), to obtain an absolute score for each
-        // node, we normalise so that the sum of the components are equal to 1.
+        // The eigenvector is a relative score of vertex importance (normalised by the norm), to obtain an absolute score for each
+        // vertex, we normalise so that the sum of the components are equal to 1.
         let sum = highest_eigenvector.sum() / self.index.as_ref().unwrap().len() as f64;
         let normalised = highest_eigenvector.unscale(sum);
 
@@ -329,18 +457,35 @@ mod tests {
 
     use super::*;
 
+    #[test]
+    fn new() {
+        let _: Graph<()> = Graph::new();
+    }
+
     #[test]
     fn insert() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         let edge = Edge::new("a", "b");
 
         assert!(graph.insert(edge.clone()));
         assert!(!graph.insert(edge));
     }
 
+    #[test]
+    fn remove() {
+        let edge = Edge::new("a", "b");
+        let uninserted_edge = Edge::new("a", "c");
+
+        let mut graph = Graph::new();
+        graph.insert(edge.clone());
+
+        assert!(graph.remove(&edge));
+        assert!(!graph.remove(&uninserted_edge));
+    }
+
     #[test]
     fn contains() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         let edge = Edge::new("a", "b");
 
         graph.insert(edge.clone());
@@ -351,7 +496,7 @@ mod tests {
 
     #[test]
     fn vertex_count() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert_eq!(graph.vertex_count(), 0);
 
         // Verify two new vertices get added when they don't yet exist in the graph.
@@ -365,7 +510,7 @@ mod tests {
 
     #[test]
     fn edge_count() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert_eq!(graph.edge_count(), 0);
 
         graph.insert(Edge::new("a", "b"));
@@ -374,7 +519,7 @@ mod tests {
 
     #[test]
     fn density() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert!(graph.density().is_nan());
 
         graph.insert(Edge::new("a", "b"));
@@ -386,7 +531,7 @@ mod tests {
 
     #[test]
     fn adjacency_matrix() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert_eq!(graph.adjacency_matrix(), dmatrix![]);
 
         graph.insert(Edge::new("a", "b"));
@@ -410,7 +555,7 @@ mod tests {
 
     #[test]
     fn degree_matrix() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert_eq!(graph.degree_matrix(), dmatrix![]);
 
         graph.insert(Edge::new("a", "b"));
@@ -434,7 +579,7 @@ mod tests {
 
     #[test]
     fn laplacian_matrix() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert_eq!(graph.laplacian_matrix(), dmatrix![]);
 
         graph.insert(Edge::new("a", "b"));
@@ -458,7 +603,7 @@ mod tests {
 
     #[test]
     fn degree_centrality_delta() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert_eq!(graph.degree_centrality_delta(), 0.0);
 
         graph.insert(Edge::new("a", "b"));
@@ -470,7 +615,7 @@ mod tests {
 
     #[test]
     fn degree_centrality() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert!(graph.degree_centrality().is_empty());
 
         // One connection, centrality measures for each vertex should be 1.
@@ -498,7 +643,7 @@ mod tests {
 
     #[test]
     fn eigenvalue_centrality() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert!(graph.eigenvalue_centrality().is_empty());
 
         // One connection, centrality measures for each vertex should be 1.
@@ -535,7 +680,7 @@ mod tests {
 
     #[test]
     fn fiedler() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
 
         let (a, b, c, d) = ("a", "b", "c", "d");
 
@@ -586,7 +731,7 @@ mod tests {
 
     #[test]
     fn clear_cache_on_insert() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         graph.insert(Edge::new("a", "b"));
 
         // The laplacian requires the computation of the index, the degree matrix and the adjacency
@@ -611,7 +756,7 @@ mod tests {
 
     #[test]
     fn vertices_from_edges() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
         assert!(graph.vertices_from_edges().is_empty());
 
         let (a, b) = ("a", "b");
@@ -627,7 +772,7 @@ mod tests {
 
     #[test]
     fn generate_index() {
-        let mut graph = Graph::default();
+        let mut graph = Graph::new();
 
         // Check for an empty graph.
         graph.generate_index();
@@ -642,12 +787,12 @@ mod tests {
 
         assert_eq!(
             graph.index.as_ref().unwrap().get_key_value(a),
-            Some((&a, &(0 as usize)))
+            Some((&a, &0))
         );
 
         assert_eq!(
             graph.index.as_ref().unwrap().get_key_value(b),
-            Some((&b, &(1 as usize)))
+            Some((&b, &1))
         );
 
         assert_eq!(graph.index.as_ref().unwrap().len(), 2);
